# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: MsgLib.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cMsgLib.proto\x1a\x19google/protobuf/any.proto\"A\n\x0bmsgEnvelope\x12\x0f\n\x07msgName\x18\x01 \x01(\t\x12!\n\x03msg\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Anyb\x06proto3')



_MSGENVELOPE = DESCRIPTOR.message_types_by_name['msgEnvelope']
msgEnvelope = _reflection.GeneratedProtocolMessageType('msgEnvelope', (_message.Message,), {
  'DESCRIPTOR' : _MSGENVELOPE,
  '__module__' : 'MsgLib_pb2'
  # @@protoc_insertion_point(class_scope:msgEnvelope)
  })
_sym_db.RegisterMessage(msgEnvelope)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MSGENVELOPE._serialized_start=43
  _MSGENVELOPE._serialized_end=108
# @@protoc_insertion_point(module_scope)
